import urllib

from .RestClient import RestClient
from .models.CreateTestRunRequest import CreateTestRunRequest

class RobotClient:
    def __init__(self, baseUri):
        self.__client = RestClient(baseUri)

        self.__isDisposed = False
    
    def createTestRun(self, scriptFilePath, workingDirectory, metadata):
        self.__throwIfDisposed()

        requestObj = CreateTestRunRequest(scriptFilePath, workingDirectory, metadata)

        testRun = self.__client.postJson("/api/v2/testruns", requestObj, "Error creating session")

        return testRun["id"]

    def getSessionInfo(self, testRunId):
        self.__throwIfDisposed()

        return self.__client.getJson(f'/api/v2/testruns/{testRunId}/info', 'Error getting session info')

    def getTestObject(self, resourceId):
        self.__throwIfDisposed()

        return self.__client.getJson('/api/v2/testobjects/' + urllib.parse.quote(resourceId), 'Get TestObject by id failed')

    def executeCommand(self, testRunId, resourceName, commandName, requestObj, errorMessagePrefix):
        self.__throwIfDisposed()

        return self.__client.postJson(f'/api/v2/testruns/{testRunId}/testobjects/{urllib.parse.quote(resourceName)}/commands/{commandName}', requestObj, errorMessagePrefix)

    def postTestResult(self, testRunId, requestObj):
        self.__throwIfDisposed()

        self.__client.post(f'/api/v2/testruns/{testRunId}/result', requestObj, 'Error setting test result')

    def tearDown(self, testRunId):
        self.__throwIfDisposed()

        self.__client.delete('/api/v2/testruns/' + testRunId, None, 'Error tearing down session')

    def dispose(self):
        self.__isDisposed = True

        self.__client = None

    def __throwIfDisposed(self):
        if self.__isDisposed is True:
            raise Exception("Cannot access a disposed object.")


class TestResult:
    def __init__(self, result, comment):
        self.__result = result.value
        self.__comment = comment
    
    @property
    def result(self):
        return self.__result
    
    @property
    def comment(self):
        return self.__comment



import jsons
import http.client

from .Errors import TestSystemError

class RestClient:
    def __init__(self, baseUri):
        self._baseUri = baseUri

    def __createConnection(self):
        return http.client.HTTPConnection(self._baseUri)

    def __get(self, relativeUrl, errorMessagePrefix):
        conn = self.__createConnection()
        try:
            response = self.__send(conn, "GET", relativeUrl, None,  errorMessagePrefix)
            return response.read().decode('utf-8')
        finally:
            conn.close()

    def getText(self, relativeUrl, errorMessagePrefix):
        text = self.__get(relativeUrl, errorMessagePrefix)
        return str(text)

    def getJson(self, relativeUrl, errorMessagePrefix):
        text = self.__get(relativeUrl, errorMessagePrefix)
        return jsons.loads(text)

    def post(self, relativeUrl, requestObj, errorMessagePrefix):
        conn = self.__createConnection()
        try:
            self.__send(conn, "POST", relativeUrl, requestObj, errorMessagePrefix)
        finally:
            conn.close()

    def postJson(self, relativeUrl, requestObj, errorMessagePrefix):
        conn = self.__createConnection()
        try:
            response = self.__send(conn, "POST", relativeUrl, requestObj, errorMessagePrefix)
            text = response.read().decode('utf-8')
            return jsons.loads(text)
        finally:
            conn.close()

    def put(self, relativeUrl, requestObj, errorMessagePrefix):
        conn = self.__createConnection()
        try:
            self.__send(conn, "PUT", relativeUrl, requestObj, errorMessagePrefix)
        finally:
            conn.close()

    def delete(self, relativeUrl, requestObj, errorMessagePrefix):
        conn = self.__createConnection()
        try:
            self.__send(conn, "DELETE", relativeUrl, requestObj, errorMessagePrefix)
        finally:
            conn.close()

    def __send(self, conn, method, relativeUrl, requestObj, errorMessagePrefix):
        headers = {"Content-Type": "application/json;charset=UTF-8"}
        if requestObj is None:
            requestBody = None
        else:
            requestBody = jsons.dumps(requestObj)

        try:
            conn.request(method, relativeUrl, requestBody, headers)
        except:
            raise Exception("Could not connect to robot")
        response = conn.getresponse()
        if response.status != 200:
            self.__throwHttpRequestError(response, errorMessagePrefix)

        return response

    def __throwHttpRequestError(self, response, message):
        if response.status == 0:
            message += ": Could not connect to robot"
            raise TestSystemError(message)
        elif response.status == 500:
            errorObj = jsons.loads(response.read())
            if "message" in errorObj:
                message += ": " + errorObj["message"]
            if "exceptionMessage" in errorObj:
                message += ": " + errorObj["exceptionMessage"]
            if self.__isSytemException(errorObj):
                raise TestSystemError(message)
        elif response.status == 409:
            message += ": All executors are busy"
            raise TestSystemError(message)
        elif response.reason:
            text = response.read()
            try:
                errorObj = jsons.loads(text)
                if "message" in errorObj:
                    message += ": " + errorObj["message"]
                if "exceptionMessage" in errorObj:
                    message += ": " + errorObj["exceptionMessage"]
                message += ": " + errorObj
            except:
                message += ": " + str(text)
        raise Exception(message)

    def __isSytemException(self, ex):
        return ex["exceptionType"] == "TestWizard.Automation.Extensibility.Exceptions.TestSystemException"



class CreateTestRunRequest:
    def __init__(self, scriptFilePath, workingDirectory, metadata):
        self.__scriptFilePath = scriptFilePath
        self.__workingDirectory = workingDirectory
        self.__tester = metadata.get("tester")
        self.__parameters = metadata.get("parameters")
        self.__resources = metadata.get("resources", {})
        self.__customProperties = metadata.get("customProperties")
        self.__outputFolder = metadata.get("outputFolder")

    @property
    def scriptFilePath(self):
        return self.__scriptFilePath

    @property
    def workingDirectory(self):
        return self.__workingDirectory

    @property
    def tester(self):
        return self.__tester

    @property
    def parameters(self):
        return self.__parameters

    @property
    def resources(self):
        return self.__resources

    @property
    def customProperties(self):
        return self.__customProperties

    @property
    def outputFolder(self):
        return self.__outputFolder
